@@ -78,6 +78,28 @@ def unit_iter_parser(inp, unit):
     return res.flatten()
 
 
+def interpolate_lin(x, y, x0):
+    p1 = poly1d(
+        polyfit(
+            x - x0,
+            y,
+            1,
+        )
+    )
+    return p1(0)
+
+
+def interpolate_log(x, y, x0):
+    p1 = poly1d(
+        polyfit(
+            np.log(x),
+            np.log(y),
+            1,
+        )
+    )
+    return np.exp(p1(np.log(x0)))
+
+
 class UnitSettings:
     SHOW_UNIT = True
     SQUEEZE = True
@@ -171,48 +193,66 @@ class BaseElement:
     def interpolate_chantler(energy, df):
         nist = df.values.astype(float).T
         arr = np.abs(nist[0] - energy)
-        idx = arr.argsort()[:2]  # find nearest data-points
         out = []
 
-        def interpolate():
-            pass
+        idx0 = arr.argsort()[0]
+
+        #Make sure one datapoint is below and one above
+        if nist[0, idx0] >= energy:
+            idx = [idx0-1, idx0]
+        else:
+            idx = [idx0, idx0+1]
 
         i = 1
+        try:
+            ans = interpolate_lin(nist[0, idx], nist[i, idx], energy)
+        except IndexError:
+            ans =np.nan
 
-        p1 = poly1d(
-            polyfit(
-                nist[0, idx],
-                nist[i, idx],
-                1,
-            )
-        )
-        out.append(p1(energy))
+        out.append(ans)
+        #print(ans)
 
         i = 2
-        """ overflow at absorption edge, no np.float128 in windows?
-        p2 = polyfit(  # nist recommends log-log for f2
-            np.log10(nist[0, idx]),
-            np.log10(nist[i, idx]),
-            1,
-        )
-        print(nist[0, idx], nist[i, idx])
-        print((Energy.dtype, p2[0].dtype, p2[1].dtype))
-        res = (lambda E, m, b: E ** m * 10 ** b)(float(Energy), float(p2[0]), float(p2[1]))
-
-        out.append(res)
-        """
-
-        p1 = poly1d(
-            polyfit(
-                nist[0, idx],
-                nist[i, idx],
-                1,
-            )
-        )
-        out.append(p1(energy))
-
+        try:
+            ans = interpolate_log(nist[0, idx], nist[i, idx], energy)
+        except IndexError:
+            ans =np.nan
+        out.append(ans)
+        #print(ans)
         return out
 
+
+    @staticmethod
+    def interpolate_hubbell(energy, df):
+        nist = df.values.astype(float).T
+        #print("energy: ", energy)
+        energy= Quantity(energy, 'MeV').value
+        arr = np.abs(nist[0] - energy)
+        out = []
+
+        srtd = arr.argsort(kind='mergesort')
+        #handle the double value case on edges
+        if (nist[0, srtd[0]]==nist[0, srtd[1]]) and (energy >= nist[0, srtd[0]]):
+            idx0 = srtd[1]
+        else:
+            idx0 = srtd[0]
+
+        # Make sure one datapoint is below and one above
+        if nist[0, idx0] >= energy:
+            idx = [idx0 - 1, idx0]
+        else:
+            idx = [idx0, idx0 + 1]
+
+        i = 1
+        try:
+            ans = interpolate_log(nist[0, idx], nist[i, idx], energy)
+        except IndexError:
+            ans =np.nan
+        out.append(ans)
+        return out
+
+
+
     @staticmethod
     def calc_q(energy, ttheta):
         energy = Quantity(energy, UnitSettings.UNIT_E)
@@ -565,11 +605,34 @@ class Element(BaseElement):
         return Quantity(ans, 'barn') if not ans == '---' else None
 
     @property
+    @squeezer()
     def mup(self):
         return self.get_mup(self._energy)
 
+
+    @property
+    @squeezer()
+    def mup_en(self):
+        return self.get_mup_en(self._energy)
+
+    def get_mup_en(self, energy):
+        res = []
+        for e in energy:
+            res.append(self._get_nist_mup_en_hubbell(e))
+        return Quantity(res, 'cm2/g')
+
+
     def get_mup(self, energy):
-        return self._get_nist_mup_chantler(energy)
+        CHANTLER_E_MAX = Quantity('4.329451E+02', 'keV')
+        energy = Quantity(energy, 'keV')
+        res = []
+        for e in energy:
+            if e <= CHANTLER_E_MAX:
+                ans = self._get_nist_mup_chantler(e)
+            else:
+                ans = self._get_nist_mup_hubbell(e)
+            res.append(ans)
+        return Quantity(res, 'cm2/g')
 
     def get_f0(self, q):
         try:
@@ -623,8 +686,18 @@ class Element(BaseElement):
         return res
 
     def _get_nist_mup_chantler(self, energy):
-        mu = self._get_nist_f1f2mu_chantler(energy)['µ/p total (cm2/g)']
-        return Quantity(float(mu), 'cm^2/g')
+        return self._get_nist_f1f2mu_chantler(energy)['µ/p total (cm2/g)']
+
+
+    def _get_nist_mup_en_hubbell(self, energy):
+        df = dabax.get_table(self.element_symbol, "nist_atten_hubbell")
+        return Element.interpolate_hubbell(energy, df[['Energy (MeV)', 'μen/ρ (cm2/g)']])
+
+    def _get_nist_mup_hubbell(self, energy):
+        df = dabax.get_table(self.element_symbol, "nist_atten_hubbell")
+        return Element.interpolate_hubbell(energy, df[['Energy (MeV)', 'μ/ρ (cm2/g)']])
+
+
 
     def _get_dabax_f0_waaskirf(self, q):
         df = dabax.get_table(self.symbol, "dabax_f0_WaasKirf")
@@ -660,10 +733,15 @@ class Element(BaseElement):
         df.columns = ['values']
         return df
 
+
+
+
+
     def _get_nist_f1f2_chantler(self, energy):
 
         df = dabax.get_table(self.element_symbol, "nist_f1f2_chantler")
 
+        #3/5CL used as relativistic correction
         rel_corr = float(dabax.get_entry(
             self.symbol,
             ["nist_f1f2_chantler", "relativistic_correction"],
@@ -676,6 +754,7 @@ class Element(BaseElement):
         res = []
         for e in energy:
             f1, f2 = Element.interpolate_chantler(e, df)
+            #equation (3), FFAST documentation
             res.append(f1 + rel_corr + nt_corr + 1j * f2)
 
         return res
@@ -1187,27 +1266,52 @@ class Compound(BaseElement):
     def mu(self):
         return self.mup * self.density if self.mup is not None else None
 
+
+    @property
+    def mu_en(self):
+        return self.mup_en * self.density if self.mup_en is not None else None
+
     @property
     @squeezer()
     def mup(self):
         return self.get_mup(self._energy)
 
 
+    @property
+    @squeezer()
+    def mup_en(self):
+        return self.get_mup_en(self._energy)
+
+    """
+    not needed
+    def get_mu_en(self, energy):
+        return self.get_mup_en(energy) * self.density if self.get_mup_en(energy) is not None else None
+
+
     def get_mu(self, energy):
         return self.get_mup(energy) * self.density if self.get_mup(energy) is not None else None
-
+    """
     def get_mup(self, energy):
+        res = 0
+        try:
+            for k in self.composition:
+                res += Elements[k].get_mup(energy) * Elements[k].atomic_mass * self.composition[k]
+        except TypeError:
+            res = None
 
-        ans = []
-        for e in energy:
-            res = 0
-            try:
-                for k in self.composition:
-                    res += Elements[k].get_mup(e) * Elements[k].atomic_mass * self.composition[k]
-            except TypeError:
-                res = None
-            ans.append(res)
-        return Quantity(ans) / self.molecular_mass
+        return Quantity(res.T.squeeze()) / self.molecular_mass
+
+
+
+    def get_mup_en(self, energy):
+        res = 0
+        try:
+            for k in self.composition:
+                res += Elements[k].get_mup_en(energy) * Elements[k].atomic_mass * self.composition[k]
+        except TypeError:
+            res = None
+
+        return Quantity(res.T.squeeze()) / self.molecular_mass
 
     @property
     def attenuation_length(self):
@@ -1360,7 +1464,7 @@ class Compound(BaseElement):
                     1
                     if submatch.group(2) == ""
                     else (
-                        int(submatch.group(2))
+                        int(float(submatch.group(2)))
                         if float(submatch.group(2)).is_integer()
                         else float(submatch.group(2))
                     )
@@ -2190,3 +2294,34 @@ Mc = Elements['Mc']
 Lv = Elements['Lv']
 Ts = Elements['Ts']
 Og = Elements['Og']
+
+Compounds = {}
+
+"""tb = dabax.dbb.table('CompoundsDabax')
+for i, cmpd in enumerate(tb):
+    #52 and above is xaamdi, but weird format
+    if i >= 51:
+        break
+    #formula = re.sub("\(|\)", "", cmpd['formula'])
+    Compounds.update({cmpd['name']: Compound(cmpd['formula'], density=cmpd['rho'])})"""
+
+
+tb = dabax.dbb.table('CompoundsHubbell')
+for cmpd in tb:
+    t = cmpd['composition']['mass']
+
+    res = []
+    els = []
+    ns = []
+    for e in t:
+        n = t[e] / Elements[e]._atomic_mass_atomic_constants().value
+        res.append(float(n))
+        els.append(e)
+    ns = np.array(res)
+    ns /= ns.min()
+
+    res = []
+    for e, n in zip(els, ns.round(decimals=4)):
+        res.append("%s%f" % (e, float(n)))
+    formula = "".join(res)
+    Compounds.update({cmpd['name']: Compound(formula, density=cmpd['rho (g/cm3)'])})
\ No newline at end of file
